import OpenSeadragon from 'openseadragon'
import type { Marker } from '@/types/virages'

export function useViewer(Viewer: Ref<OpenSeadragon.Viewer>) {
  const zoomTo = (zoomLevel: number) => {
    Viewer.value.viewport.zoomTo(zoomLevel, undefined, false)
  }

  const goHome = () => {
    Viewer.value.viewport.goHome(false)
  }

  const enableMouseNav = () => {
    Viewer.value.setMouseNavEnabled(true)
  }

  const disableMouseNav = () => {
    Viewer.value.setMouseNavEnabled(false)
    // Viewer.value.gestureSettingsMouse.dragToPan = false
  }

  const injectMarker = (marker: Marker, overlay: HTMLElement) => {
    Viewer.value.addOverlay({
      element: overlay,
      location: new OpenSeadragon.Point(marker.point.x, marker.point.y),
    })
  }

  const addHandlerOnce = (eventName: string, handler: (event) => void) => {
    return Viewer.value.addHandler(eventName, handler)
  }

  const addClickHandler = (handler: (event) => void) => {
    return Viewer.value.addHandler('canvas-click', handler)
  }
  // const addClickHandler = (handler(event): (event: Event) => typeof handler) => {
  //   return Viewer.value.addHandler('canvas-click', handler)
  // }

  const removeClickHandler = (handler) => {
    Viewer.value.removeHandler('canvas-click', handler)
  }

  const getZoom = () => {
    return Viewer.value.viewport.getZoom()
  }

  const getBounds = () => {
    return Viewer.value.viewport.getBounds()
  }

  const pointFromPixel = (position) => {
    const point = Viewer.value.viewport.pointFromPixel(position)
    return new OpenSeadragon.Point(point.x, point.y)
  }

  const defaultZoomLevel = (level: number) => {
    Viewer.value.defaultZoomLevel = level
  }

  const viewerClickAction = (action) => {
    action()
    console.log('click action for viewer')
  }

  const selectedMarkerClickAction = (action) => {
    action()
    console.log('click action for selected marker')
  }

  return {
    zoomTo,
    goHome,
    enableMouseNav,
    disableMouseNav,
    injectMarker,
    addClickHandler,
    addHandlerOnce,
    removeClickHandler,
    getZoom,
    getBounds,
    pointFromPixel,
    defaultZoomLevel,
  }
}
